@@ -9,7 +9,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tkMacOSXInt.h,v 1.3.2.7 2005/11/30 01:02:55 hobbs Exp $
+ * RCS: @(#) $Id: tkMacOSXInt.h,v 1.3.2.14 2006/09/11 14:41:16 das Exp $
  */
 
 #ifndef _TKMACINT
@@ -19,6 +19,10 @@
 #include "tkInt.h"
 #endif
 
+#define TextStyle MacTextStyle
+#include <Carbon/Carbon.h>
+#undef TextStyle
+
 /*
  * Include platform specific public interfaces.
  */
@@ -27,20 +31,14 @@
 #include "tkMacOSX.h"
 #endif
 
-#ifndef _TKPORT
-#include "tkPort.h"
-#endif
-
-#include <Carbon/Carbon.h>
-
 struct TkWindowPrivate {
     TkWindow *winPtr;     	/* Ptr to tk window or NULL if Pixmap */
     CGrafPtr  grafPtr;
     ControlRef rootControl;
     int xOff;	       		/* X offset from toplevel window */
     int yOff;		       	/* Y offset from toplevel window */
-    RgnHandle clipRgn;		/* Visable region of window */
-    RgnHandle aboveClipRgn;	/* Visable region of window & it's children */
+    RgnHandle clipRgn;		/* Visible region of window */
+    RgnHandle aboveClipRgn;	/* Visible region of window & it's children */
     int referenceCount;		/* Don't delete toplevel until children are
 				 * gone. */
     struct TkWindowPrivate *toplevel;	/* Pointer to the toplevel
@@ -68,14 +66,14 @@ typedef struct TkMacOSXWindowList {
 #define TK_CLIP_INVALID		2
 #define TK_HOST_EXISTS		4
 #define TK_DRAWN_UNDER_MENU	8
-#define TK_MAPPED_IN_PARENT	16
 
 /*
  * I am reserving TK_EMBEDDED = 0x100 in the MacDrawable flags
  * This is defined in tk.h. We need to duplicate the TK_EMBEDDED flag in the
- * TkWindow structure for the window,  but in the MacWin.  This way we can still tell
- * what the correct port is after the TKWindow  structure has been freed.  This 
- * actually happens when you bind destroy of a toplevel to Destroy of a child.
+ * TkWindow structure for the window, but in the MacWin.  This way we can
+ * still tell what the correct port is after the TKWindow structure has been
+ * freed.  This actually happens when you bind destroy of a toplevel to
+ * Destroy of a child.
  */
 
 /*
@@ -94,7 +92,7 @@ typedef struct {
 	Tk_MacOSXEmbedGetOffsetInParentProc *getOffsetProc;
 } TkMacOSXEmbedHandler;
 
-extern TkMacOSXEmbedHandler *gMacEmbedHandler;
+MODULE_SCOPE TkMacOSXEmbedHandler *gMacEmbedHandler;
 
 /*
  * Defines used for TkMacOSXInvalidateWindow
@@ -106,14 +104,14 @@ extern TkMacOSXEmbedHandler *gMacEmbedHandler;
 /*
  * Accessor for the privatePtr flags field for the TK_HOST_EXISTS field
  */
- 
+
 #define TkMacOSXHostToplevelExists(tkwin) \
     (((TkWindow *) (tkwin))->privatePtr->toplevel->flags & TK_HOST_EXISTS)
 
 /*
  * Defines use for the flags argument to TkGenWMConfigureEvent.
  */
- 
+
 #define TK_LOCATION_CHANGED	1
 #define TK_SIZE_CHANGED		2
 #define TK_BOTH_CHANGED		3
@@ -122,43 +120,50 @@ extern TkMacOSXEmbedHandler *gMacEmbedHandler;
  * Variables shared among various Mac Tk modules but are not
  * exported to the outside world.
  */
- 
+
 /*
  * Globals shared among Macintosh Tk
  */
- 
-extern MenuHandle tkAppleMenu;		/* Handle to the Apple Menu */
-extern MenuHandle tkFileMenu;		/* Handles to menus */
-extern MenuHandle tkEditMenu;		/* Handles to menus */
-extern RgnHandle tkMenuCascadeRgn;	/* A region to clip with. */
-extern int tkUseMenuCascadeRgn;		/* If this is 1, clipping code
+
+MODULE_SCOPE MenuHandle tkCurrentAppleMenu; /* Handle to current Apple Menu */
+MODULE_SCOPE MenuHandle tkAppleMenu;	/* Handle to default Apple Menu */
+MODULE_SCOPE MenuHandle tkFileMenu;	/* Handles to menus */
+MODULE_SCOPE MenuHandle tkEditMenu;	/* Handles to menus */
+MODULE_SCOPE RgnHandle tkMenuCascadeRgn;/* A region to clip with. */
+MODULE_SCOPE int tkUseMenuCascadeRgn;	/* If this is 1, clipping code
 					 * should intersect tkMenuCascadeRgn
 					 * before drawing occurs.
 					 * tkMenuCascadeRgn will only
 					 * be valid when the value of this
 					 * variable is 1. */
-extern int tkPictureIsOpen;	/* If this is 1, we are drawing to a picture
-				 * The clipping should then be done relative
-				 * to the bounds of the picture rather than
-				 * the window As of OS X.0.4, something is
-				 * seriously wrong: The clipping bounds only
-				 * seem to work if the top,left values are 0,0
-				 * The destination rectangle for CopyBits
-				 * should also have top,left values of 0,0
-				 */
-extern TkMacOSXWindowList *tkMacOSXWindowListPtr;
+MODULE_SCOPE int tkPictureIsOpen;	/* If this is 1, we are drawing to a
+					 * picture The clipping should then be
+					 * done relative to the bounds of the
+					 * picture rather than the window. As
+					 * of OS X.0.4, something is seriously
+					 * wrong: The clipping bounds only
+					 * seem to work if the top,left values
+					 * are 0,0 The destination rectangle
+					 * for CopyBits should also have
+					 * top,left values of 0,0
+					 */
+MODULE_SCOPE TkMacOSXWindowList *tkMacOSXWindowListPtr;
 					/* The list of toplevels */
 
-extern Tcl_Encoding TkMacOSXCarbonEncoding;
+MODULE_SCOPE Tcl_Encoding TkMacOSXCarbonEncoding;
 
-extern int TkMacOSXUseAntialiasedText(Tcl_Interp *interp, int enable);
-extern void TkMacOSXInitCarbonEvents(Tcl_Interp *interp);
-extern int TkMacOSXInitCGDrawing(Tcl_Interp *interp, int enable, int antiAlias);
-extern void TkMacOSXDefaultStartupScript(void);
-extern int TkMacOSXGenerateFocusEvent( Window window, int activeFlag);
-extern WindowClass TkMacOSXWindowClass(TkWindow *winPtr);
-extern void* TkMacOSXGetNamedSymbol(const char* module, const char* symbol);
+MODULE_SCOPE void TkMacOSXDisplayChanged(Display *display);
+MODULE_SCOPE int TkMacOSXUseAntialiasedText(Tcl_Interp *interp, int enable);
+MODULE_SCOPE void TkMacOSXInitCarbonEvents(Tcl_Interp *interp);
+MODULE_SCOPE int TkMacOSXInitCGDrawing(Tcl_Interp *interp, int enable, int antiAlias);
+MODULE_SCOPE void TkMacOSXInitKeyboard(Tcl_Interp *interp);
+MODULE_SCOPE void TkMacOSXDefaultStartupScript(void);
+MODULE_SCOPE int TkMacOSXGenerateFocusEvent( Window window, int activeFlag);
+MODULE_SCOPE WindowClass TkMacOSXWindowClass(TkWindow *winPtr);
+MODULE_SCOPE int TkMacOSXIsWindowZoomed(TkWindow *winPtr);
+MODULE_SCOPE int TkGenerateButtonEventForXPointer(Window window);
 
+MODULE_SCOPE void* TkMacOSXGetNamedSymbol(const char* module, const char* symbol);
 /* Macro to abstract common use of TkMacOSXGetNamedSymbol to initialize named symbols */
 #define TkMacOSXInitNamedSymbol(module, ret, symbol, ...) \
     static ret (* symbol)(__VA_ARGS__) = (void*)(-1L); \
@@ -166,8 +171,6 @@ extern void* TkMacOSXGetNamedSymbol(const char* module, const char* symbol);
         symbol = TkMacOSXGetNamedSymbol(STRINGIFY(module), STRINGIFY(_##symbol));\
     }
 
-extern void TkMacOSXDisplayChanged(Display *display);
-
 #include "tkIntPlatDecls.h"
 
 #endif /* _TKMACINT */
