@@ -6,26 +6,17 @@
  *
  * Copyright (c) 1995-1996 Sun Microsystems, Inc.
  * Copyright 2001, Apple Computer, Inc.
+ * Copyright (c) 2005-2006 Daniel A. Steffen <das@users.sourceforge.net>
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tkMacOSXMenus.c,v 1.2.2.6 2005/11/27 06:53:36 das Exp $
+ * RCS: @(#) $Id: tkMacOSXMenus.c,v 1.2.2.12 2006/08/18 07:47:25 das Exp $
  */
 
-#include "tk.h"
-#include "tkInt.h"
 #include "tkMacOSXInt.h"
 
-/*
- * The define Status defined by Xlib.h conflicts with the function Status
- * defined by Devices.h.  We undefine it here to compile.
- */
-#undef Status
-#include <Carbon/Carbon.h>
-
-#if !defined(MAC_OS_X_VERSION_10_3) || \
-        (MAC_OS_X_VERSION_MAX_ALLOWED < MAC_OS_X_VERSION_10_3)
+#if MAC_OS_X_VERSION_MAX_ALLOWED < 1030
     /* Define constants only available on Mac OS X 10.3 or later */
     #define kMenuAttrDoNotUseUserCommandKeys (1 << 7)
 #endif
@@ -201,7 +192,16 @@ TkMacOSXInitMenus(
     if (TkMacOSXUseMenuID(kHMHelpMenuID) != TCL_OK) {
 	Tcl_Panic("Help menu ID %s is already in use!", kHMHelpMenuID);
     }
-    
+
+    /*
+     * Workaround a Carbon bug with kHICommandPreferences: the first call to
+     * IsMenuKeyEvent returns false for the preferences menu item key shorcut
+     * event (even if the corresponding menu item is dynamically enabled by a
+     * kEventCommandUpdateStatus handler), unless the kHICommandPreferences
+     * menu item has previously been enabled manually. [Bug 1481503]
+     */
+    EnableMenuCommand(NULL, kHICommandPreferences);
+
     DrawMenuBar();
     return;
 }
@@ -228,7 +228,7 @@ GenerateEditEvent(
     int flag)
 {
     XVirtualEvent event;
-    Point where;
+    int x, y;
     Tk_Window tkwin;
     Window window;
     TkDisplay *dispPtr;
@@ -241,6 +241,7 @@ GenerateEditEvent(
         return;
     }
 
+    bzero(&event, sizeof(XVirtualEvent));
     event.type = VirtualEvent;
     event.serial = Tk_Display(tkwin)->request;
     event.send_event = false;
@@ -251,12 +252,8 @@ GenerateEditEvent(
     event.time = TkpGetMS();
     
     XQueryPointer(NULL, None, NULL, NULL,
-            &event.x_root, &event.y_root, NULL, NULL, &event.state);
-    where.h = event.x_root;
-    where.v = event.y_root;
-    GlobalToLocal(&where);
-    tkwin = Tk_TopCoordsToWindow(tkwin, where.h, where.v, 
-            &event.x, &event.y);
+            &event.x_root, &event.y_root, &x, &y, &event.state);
+    tkwin = Tk_TopCoordsToWindow(tkwin, x, y, &event.x, &event.y);
     event.same_screen = true;
 
     switch (flag) {
@@ -316,3 +313,8 @@ SourceDialog()
         Tcl_BackgroundError(gInterp);
     }           
 }
+/*
+ * Local Variables:
+ * coding: macintosh;
+ * End:
+ */
