@@ -5,7 +5,7 @@
  *	on MacOS X.
  *
  * Copyright 2001, Apple Computer, Inc.
- * Copyright (c) 2005 Daniel A. Steffen <das@users.sourceforge.net>
+ * Copyright (c) 2005-2006 Daniel A. Steffen <das@users.sourceforge.net>
  *
  * See the file "license.terms" for information on usage and redistribution of
  * this file, and for a DISCLAIMER OF ALL WARRANTIES.
@@ -54,19 +54,15 @@
  *      software in accordance with the terms specified in this
  *      license.
  *
- * RCS: @(#) $Id: tkMacOSXMouseEvent.c,v 1.6.2.9 2005/11/27 02:36:46 das Exp $
+ * RCS: @(#) $Id: tkMacOSXMouseEvent.c,v 1.6.2.17 2006/09/10 17:07:36 das Exp $
  */
 
-#include "tkInt.h"
-#include <X11/X.h>
-#include <X11/Xlib.h>
-#include "tkMacOSXEvent.h"
 #include "tkMacOSXInt.h"
-#include "tkPort.h"
+#include "tkMacOSXWm.h"
+#include "tkMacOSXEvent.h"
 #include "tkMacOSXDebug.h"
 
-#if !defined(MAC_OS_X_VERSION_10_3) || \
-        (MAC_OS_X_VERSION_MAX_ALLOWED < MAC_OS_X_VERSION_10_3)
+#if MAC_OS_X_VERSION_MAX_ALLOWED < 1030
     /* Define constants only available on Mac OS X 10.3 or later */
     #define kEventParamWindowPartCode 'wpar'
     #define typeWindowPartCode        'wpar'
@@ -121,15 +117,17 @@ static void TkMacOSXSetEatButtonUp(int f);
  *----------------------------------------------------------------------
  */
 
-int
+MODULE_SCOPE int
 TkMacOSXProcessMouseEvent(TkMacOSXEvent *eventPtr, MacEventStatus * statusPtr)
 {
     Tk_Window	   tkwin;
     Point	   where, where2;
     int		   xOffset, yOffset, result;
     TkDisplay *	   dispPtr;
-    OSStatus	    status;
+    OSStatus	   status;
     MouseEventData mouseEventData, * medPtr = &mouseEventData;
+    ProcessSerialNumber frontPsn, ourPsn = {0, kCurrentProcess};
+    Boolean	   isFrontProcess = true;
 
     switch (eventPtr->eKind) {
 	case kEventMouseUp:
@@ -139,7 +137,7 @@ TkMacOSXProcessMouseEvent(TkMacOSXEvent *eventPtr, MacEventStatus * statusPtr)
 	case kEventMouseWheelMoved:
 	    break;
 	default:
-	    return 0;
+	    return false;
 	    break;
     }
     status = GetEventParameter(eventPtr->eventRef, 
@@ -167,10 +165,33 @@ TkMacOSXProcessMouseEvent(TkMacOSXEvent *eventPtr, MacEventStatus * statusPtr)
     }
     medPtr->window = TkMacOSXGetXWindow(medPtr->whichWin);
     if (medPtr->whichWin != NULL && medPtr->window == None) {
-	return 0;
+	return false;
+    }
+    if (eventPtr->eKind == kEventMouseDown) {
+	if (IsWindowPathSelectEvent(medPtr->whichWin, eventPtr->eventRef)) {
+	    status = WindowPathSelect(medPtr->whichWin, NULL, NULL);
+	    return false;
+	}
+	if (medPtr->windowPart == inProxyIcon) {
+	    status = TrackWindowProxyDrag(medPtr->whichWin, where);
+	    if (status == errUserWantsToDragWindow) {
+		medPtr->windowPart = inDrag;
+	    } else {
+		return false;
+	    }
+	}
+    }
+    status = GetFrontProcess(&frontPsn);
+    if (status == noErr) {
+	SameProcess(&frontPsn, &ourPsn, &isFrontProcess);
+    }
+    if (isFrontProcess) {
+	medPtr->state = ButtonModifiers2State(GetCurrentEventButtonState(),
+		GetCurrentEventKeyModifiers());
+    } else {
+	medPtr->state = ButtonModifiers2State(GetCurrentButtonState(),
+		GetCurrentKeyModifiers());
     }
-    medPtr->state = ButtonModifiers2State(GetCurrentEventButtonState(),
-            GetCurrentEventKeyModifiers());
     medPtr->global = where;
     status = GetEventParameter(eventPtr->eventRef, 
 	    kEventParamWindowMouseLocation,
@@ -206,19 +227,29 @@ TkMacOSXProcessMouseEvent(TkMacOSXEvent *eventPtr, MacEventStatus * statusPtr)
 	}
 	return GenerateButtonEvent(medPtr);
     }
+    if (eventPtr->eKind == kEventMouseDown) {
+	TkMacOSXSetEatButtonUp(false);
+    }
     if (eventPtr->eKind == kEventMouseWheelMoved) {
 	status = GetEventParameter(eventPtr->eventRef,
-		kEventParamMouseWheelDelta,
-		typeLongInteger, NULL,
+		kEventParamMouseWheelDelta, typeLongInteger, NULL,
 		sizeof(long), NULL, &medPtr->delta);
 	if (status != noErr ) {
 #ifdef TK_MAC_DEBUG
-	     fprintf (stderr,
-		 "Failed to retrieve mouse wheel delta, %d\n", (int) status);
+	    fprintf (stderr,
+		"Failed to retrieve mouse wheel delta, %d\n", (int) status);
 #endif
-	     statusPtr->err = 1;
-	     return false;
-	 }
+	    statusPtr->err = 1;
+	    return false;
+	} else {
+	    EventMouseWheelAxis axis;
+	    status = GetEventParameter(eventPtr->eventRef,
+		    kEventParamMouseWheelAxis, typeMouseWheelAxis, NULL,
+		    sizeof(EventMouseWheelAxis), NULL, &axis);
+	    if (status == noErr && axis == kEventMouseWheelAxisX) {
+		 medPtr->state |= ShiftMask;
+	    }
+	}
     }
 
     dispPtr = TkGetDisplayList();
@@ -226,11 +257,15 @@ TkMacOSXProcessMouseEvent(TkMacOSXEvent *eventPtr, MacEventStatus * statusPtr)
 
     if (eventPtr->eKind != kEventMouseDown) {
 	/*
-	 * MouseMoved, MouseDragged or kEventMouseWheelMoved 
+	 * MouseMoved, MouseDragged or MouseWheelMoved 
 	 */
 
 	if (eventPtr->eKind == kEventMouseWheelMoved) {
-	    return GenerateMouseWheelEvent(medPtr);
+	    int res = GenerateMouseWheelEvent(medPtr);
+	    if (res) {
+		 statusPtr->stopProcessing = 1;
+	    }
+	    return res;
 	} else {
 	    return GeneratePollingEvents(medPtr);
 	}
@@ -245,14 +280,6 @@ TkMacOSXProcessMouseEvent(TkMacOSXEvent *eventPtr, MacEventStatus * statusPtr)
 	 * or else it will mess up some Tk scripts.
 	 */
 
-	ProcessSerialNumber frontPsn, ourPsn = {0, kCurrentProcess};
-	Boolean		    isFrontProcess = true;
-
-	status = GetFrontProcess(&frontPsn);
-	if (status == noErr) {
-	    SameProcess(&frontPsn, &ourPsn, &isFrontProcess);
-	}
-
 	if (!(TkpIsWindowFloating(medPtr->whichWin))
 		&& (medPtr->whichWin != medPtr->activeNonFloating
 		|| !isFrontProcess)) {
@@ -284,70 +311,80 @@ TkMacOSXProcessMouseEvent(TkMacOSXEvent *eventPtr, MacEventStatus * statusPtr)
 	    }
 
 	    /*
-	     * Clicks in the stoplights on a MacOS X title bar are processed
-	     * directly even for background windows.  Do that here.
+	     * Clicks in the titlebar widgets are handled without bringing the
+	     * window forward.
 	     */
             if ((result = HandleWindowTitlebarMouseDown(medPtr, tkwin)) != -1) {
                 return result;
-            } else {
-                TkMacOSXSetEatButtonUp(true);
-                BringWindowForward(medPtr->whichWin, isFrontProcess);
-                return false;
+            } else
+		    /*
+		     * Only windows with the kWindowNoActivatesAttribute can
+		     * receive mouse events in the background.
+		     */
+		    if (!(((TkWindow *)tkwin)->wmInfoPtr->attributes &
+			    kWindowNoActivatesAttribute)) {
+		/*
+		 * Allow background window dragging & growing with Command down
+		 */
+                if (!((medPtr->windowPart == inDrag ||
+			medPtr->windowPart == inGrow) &&
+			medPtr->state & Mod1Mask)) {
+		    TkMacOSXSetEatButtonUp(true);
+		    BringWindowForward(medPtr->whichWin, isFrontProcess);
+                }
+                /*
+                 * Allow dragging & growing of windows that were/are in the
+                 * background.
+                 */
+                if (!(medPtr->windowPart == inDrag ||
+			medPtr->windowPart == inGrow)) {
+		    return false;
+                }
             }
-	}
-    }
-
-    if ((result = HandleWindowTitlebarMouseDown(medPtr, tkwin)) != -1) {
-        return result;
-    }
-    switch (medPtr->windowPart) {
-	case inDrag: {
-	    CGrafPtr saveWorld;
-	    GDHandle saveDevice;
-	    GWorldPtr dstPort;
-
-	    GetGWorld(&saveWorld, &saveDevice);
-	    dstPort = TkMacOSXGetDrawablePort(Tk_WindowId(tkwin));
-	    SetGWorld(dstPort, NULL);
-
-	    DragWindow(medPtr->whichWin, where, NULL);
-	    where2.h = where2.v = 0;
-	    LocalToGlobal(&where2);
-	    if (EqualPt(where, where2)) {
-		SetGWorld (saveWorld, saveDevice);
-		return false;
+	} else {
+	    if ((result = HandleWindowTitlebarMouseDown(medPtr, tkwin)) != -1) {
+		return result;
 	    }
-	    TkMacOSXWindowOffset(medPtr->whichWin, &xOffset, &yOffset);
-	    where2.h -= xOffset;
-	    where2.v -= yOffset;
-	    TkGenWMConfigureEvent(tkwin, where2.h, where2.v,
-		    -1, -1, TK_LOCATION_CHANGED);
-	    SetGWorld(saveWorld, saveDevice);
-	    return true;
-	    break;
 	}
-	case inContent:
-	    return GenerateButtonEvent(medPtr);
-	    break;
-	case inGrow:
-	    /*
-	     * Generally the content region is the domain of Tk
-	     * sub-windows.  However, one exception is the grow	  
-	     * region.	A button down in this area will be handled
-	     * by the window manager.  Note: this means that Tk
-	     * may not get button down events in this area!
-	     */
-	    if (TkMacOSXGrowToplevel(medPtr->whichWin, where) == true) {
+	switch (medPtr->windowPart) {
+	    case inDrag:
+		SetPort(GetWindowPort(medPtr->whichWin));
+		DragWindow(medPtr->whichWin, where, NULL);
+		where2.h = where2.v = 0;
+		LocalToGlobal(&where2);
+		if (EqualPt(where, where2)) {
+		    return false;
+		}
+		TkMacOSXWindowOffset(medPtr->whichWin, &xOffset, &yOffset);
+		where2.h -= xOffset;
+		where2.v -= yOffset;
+		TkGenWMConfigureEvent(tkwin, where2.h, where2.v,
+			-1, -1, TK_LOCATION_CHANGED);
 		return true;
-	    } else {
+		break;
+	    case inGrow:
+		/*
+		 * Generally the content region is the domain of Tk
+		 * sub-windows.  However, one exception is the grow	  
+		 * region.	A button down in this area will be handled
+		 * by the window manager.  Note: this means that Tk
+		 * may not get button down events in this area!
+		 */
+		if (TkMacOSXGrowToplevel(medPtr->whichWin, where) == true) {
+		    return true;
+		} else {
+		    return GenerateButtonEvent(medPtr);
+		}
+		break;
+	    case inContent:
 		return GenerateButtonEvent(medPtr);
-	    }
-	    break;
-	default:
-	    return false;
-	    break;
+		break;
+	    default:
+		return false;
+		break;
+	}
     }
-    return 0;
+    return false;
 }
 
 /*
@@ -384,7 +421,7 @@ HandleWindowTitlebarMouseDown(MouseEventData * medPtr, Tk_Window tkwin)
         case inCollapseBox:
             if (TrackBox(medPtr->whichWin, medPtr->global, medPtr->windowPart)) {
                 if (tkwin) {
-                    TkpWmSetState((TkWindow *)tkwin, IconicState);;
+                    TkpWmSetState((TkWindow *)tkwin, IconicState);
                     result = 1;
                 }
             }
@@ -534,7 +571,6 @@ static int
 GenerateMouseWheelEvent(MouseEventData * medPtr)
 {
     Tk_Window tkwin, rootwin, grabWin;
-    int local_x, local_y;
     TkDisplay *dispPtr;
     TkWindow  *winPtr;
     XEvent     xEvent;
@@ -550,7 +586,7 @@ GenerateMouseWheelEvent(MouseEventData * medPtr)
 	} else {
 	    tkwin = Tk_TopCoordsToWindow(rootwin, 
 		    medPtr->local.h, medPtr->local.v, 
-		    &local_x, &local_y);
+		    &xEvent.xbutton.x, &xEvent.xbutton.y);
 	}
     }
     
@@ -565,12 +601,14 @@ GenerateMouseWheelEvent(MouseEventData * medPtr)
 	tkwin = grabWin;
     }
     if (!tkwin) {
-       return true;
+       return false;
     }
     winPtr = (TkWindow *) tkwin;
     xEvent.type = MouseWheelEvent;
     xEvent.xkey.keycode = medPtr->delta;
-    xEvent.xbutton.state = TkMacOSXButtonKeyState();
+    xEvent.xbutton.x_root = medPtr->global.h;
+    xEvent.xbutton.y_root = medPtr->global.v;
+    xEvent.xbutton.state = medPtr->state;
     xEvent.xany.serial = LastKnownRequestProcessed(winPtr->display);
     xEvent.xany.send_event = false;
     xEvent.xany.display = winPtr->display;
@@ -642,12 +680,22 @@ unsigned int
 TkMacOSXButtonKeyState()
 {
     UInt32 buttonState = 0, keyModifiers;
-    EventRef ev = GetCurrentEvent();
+    Boolean isFrontProcess = false;
+    
+    if (GetCurrentEvent()) {
+	ProcessSerialNumber frontPsn, ourPsn = {0, kCurrentProcess};
+	OSStatus status = GetFrontProcess(&frontPsn);
+	if (status == noErr) {
+	    SameProcess(&frontPsn, &ourPsn, &isFrontProcess);
+	}
+    }
     
     if (!gEatButtonUp) {
-	buttonState = ev ? GetCurrentEventButtonState() : GetCurrentButtonState();
+	buttonState = isFrontProcess ? GetCurrentEventButtonState() :
+		GetCurrentButtonState();
     }
-    keyModifiers = ev ? GetCurrentEventKeyModifiers() : GetCurrentKeyModifiers();
+    keyModifiers = isFrontProcess ? GetCurrentEventKeyModifiers() :
+	    GetCurrentKeyModifiers();
     
     return ButtonModifiers2State(buttonState, keyModifiers);
 }
@@ -733,24 +781,69 @@ XQueryPointer(
     int* win_y_return,
     unsigned int* mask_return)
 {
-    if (root_x_return && root_y_return) {
-	Point where;
-	EventRef ev;
+    int getGlobal = (root_x_return && root_y_return);
+    int getLocal = (win_x_return && win_y_return);
+
+    if (getGlobal || getLocal) {
+	Point where, local;
 	OSStatus status;
+	int gotMouseLoc = 0;
+	EventRef ev = GetCurrentEvent();
 
-	if ((ev = GetCurrentEvent())) {
+	if (ev && getLocal) {
 	    status = GetEventParameter(ev, 
-		    kEventParamMouseLocation,
+		    kEventParamWindowMouseLocation,
 		    typeQDPoint, NULL, 
-		    sizeof(where), NULL,
-		    &where);
+		    sizeof(Point), NULL,
+		    &local);
+	    gotMouseLoc = (status == noErr);
 	}
-	if (!ev || status != noErr) {
-	    GetGlobalMouse(&where);
+	if (getGlobal || !gotMouseLoc) {
+	    if (ev) {
+		status = GetEventParameter(ev, 
+			kEventParamMouseLocation,
+			typeQDPoint, NULL, 
+			sizeof(Point), NULL,
+			&where);
+	    }
+	    if (!ev || status != noErr) {
+		GetGlobalMouse(&where);
+	    }
+	}
+	if (getLocal) {
+	    WindowRef whichWin;
+	    if (ev) {
+		status = GetEventParameter(ev, 
+			kEventParamWindowRef,
+			typeWindowRef, NULL, 
+			sizeof(WindowRef), NULL,
+			&whichWin);
+	    }
+	    if (!ev || status != noErr) {
+		FindWindow(where, &whichWin);
+	    }
+	    if (gotMouseLoc) {
+		if (whichWin) {
+		    Rect widths;
+		    GetWindowStructureWidths(whichWin, &widths);
+		    local.h -= widths.left;
+		    local.v -= widths.top;	   
+		}
+	    } else {
+		local = where;
+		if (whichWin) {
+		    QDGlobalToLocalPoint(GetWindowPort(whichWin), &local);
+		}
+	    }
+	}
+	if (getGlobal) {
+	    *root_x_return = where.h;
+	    *root_y_return = where.v;
+	}
+	if (getLocal) {
+	    *win_x_return = local.h;
+	    *win_y_return = local.v;
 	}
-
-	*root_x_return = where.h;
-	*root_y_return = where.v;
     }
     if (mask_return) {
 	*mask_return = TkMacOSXButtonKeyState();
@@ -758,6 +851,44 @@ XQueryPointer(
     return True;
 }
 
+/*
+ *----------------------------------------------------------------------
+ *
+ * TkGenerateButtonEventForXPointer --
+ *
+ *	  This procedure generates an X button event for the current
+ *	  pointer state as reported by XQueryPointer().
+ *
+ * Results:
+ *	  True if event(s) are generated - false otherwise.
+ *
+ * Side effects:
+ *	  Additional events may be place on the Tk event queue.
+ *	  Grab state may also change.
+ *
+ *----------------------------------------------------------------------
+ */
+
+MODULE_SCOPE int
+TkGenerateButtonEventForXPointer(
+    Window window)	  /* X Window containing button event. */
+{
+    MouseEventData med;
+    int global_x, global_y, local_x, local_y;
+    
+    bzero(&med, sizeof(MouseEventData));
+    XQueryPointer(NULL, None, NULL, NULL, &global_x, &global_y,
+	    &local_x, &local_y, &med.state);
+    med.global.h = global_x;
+    med.global.v = global_y;
+    med.local.h = local_x;
+    med.local.v = local_y;
+    med.window = window;
+    med.activeNonFloating = ActiveNonFloatingWindow();
+
+    return GenerateButtonEvent(&med);
+}
+
 /*
  *----------------------------------------------------------------------
  *
@@ -817,7 +948,7 @@ TkGenerateButtonEvent(
  *----------------------------------------------------------------------
  */
 
-static int
+static int
 GenerateButtonEvent(MouseEventData * medPtr)
 {
     Tk_Window tkwin;
@@ -885,8 +1016,9 @@ GenerateToolbarButtonEvent(MouseEventData * medPtr)
     if (!tkwin) {
        return true;
     }
-
     winPtr = (TkWindow *)tkwin;
+
+    bzero(&event, sizeof(XVirtualEvent));
     event.type = VirtualEvent;
     event.serial = LastKnownRequestProcessed(winPtr->display);
     event.send_event = false;
@@ -895,13 +1027,12 @@ GenerateToolbarButtonEvent(MouseEventData * medPtr)
     event.root = XRootWindow(winPtr->display, 0);
     event.subwindow = None;
     event.time = TkpGetMS();
-
     event.x_root = medPtr->global.h;
     event.y_root = medPtr->global.v;
     event.state = medPtr->state;
     event.same_screen = true;
     event.name = Tk_GetUid("ToolbarButton");
-    Tk_QueueWindowEvent((XEvent *) &event, TCL_QUEUE_TAIL);
 
+    Tk_QueueWindowEvent((XEvent *) &event, TCL_QUEUE_TAIL);
     return true;
 }
