@@ -11,16 +11,15 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tkMacOSXScrlbr.c,v 1.5.2.6 2005/10/18 00:41:19 hobbs Exp $
+ * RCS: @(#) $Id: tkMacOSXScrlbr.c,v 1.5.2.10 2006/08/24 05:22:33 das Exp $
  */
 
-#include "tkScrollbar.h"
 #include "tkMacOSXInt.h"
-#include "tclInt.h"
-
-#include <Carbon/Carbon.h>
-
+#include "tkScrollbar.h"
 #include "tkMacOSXDebug.h"
+
+#include "tclInt.h" /* for TclServiceIdle() */
+
 /*
  * The following definitions should really be in MacOS
  * header files.  They are included here as this is the only
@@ -791,8 +790,7 @@ ScrollbarBindProc(
     if (eventPtr->type == ButtonPress) {
 	Point where;
 	Rect bounds;
-	int part, x, y, dummy;
-	unsigned int state;
+	int part;
         CGrafPtr saveWorld;
         GDHandle saveDevice;
         GWorldPtr destPort;
@@ -877,10 +875,8 @@ ScrollbarBindProc(
          * The TrackControl call will "eat" the ButtonUp event.  We now
          * generate a ButtonUp event so Tk will unset implicit grabs etc.
          */
-        XQueryPointer(NULL, None, &window, &window, &x,
-            &y, &dummy, &dummy, &state);
         window = Tk_WindowId(scrollPtr->tkwin);
-        TkGenerateButtonEvent(x, y, window, state);
+        TkGenerateButtonEventForXPointer(window);
 
         SetGWorld(saveWorld, saveDevice);
     }
@@ -972,9 +968,6 @@ UpdateControlValues(
      * flicker.  To avoid this we adjust the control record directly.  The
      * Draw1Control command appears to just draw where ever the control says to
      * draw so this seems right.
-     *
-     * NOTE: changing the control record directly may not work when
-     * Apple releases the Copland version of the MacOS (or when hell is cold).
      */
 
     contrlRect.left   = macDraw->xOff + scrollPtr->inset;
@@ -982,8 +975,6 @@ UpdateControlValues(
     contrlRect.right  = macDraw->xOff + Tk_Width(tkwin) - scrollPtr->inset;
     contrlRect.bottom = macDraw->yOff + Tk_Height(tkwin) - scrollPtr->inset;
 
-    SetControlBounds(macScrollPtr->sbHandle, &contrlRect );
-
     /*
      * To make Tk applications look more like Macintosh applications without 
      * requiring additional work by the Tk developer we do some cute tricks.
@@ -1003,7 +994,6 @@ UpdateControlValues(
     if ( portRect.left == contrlRect.left ) {
         if (macScrollPtr->macFlags & AUTO_ADJUST) {
             contrlRect.left--;
-            SetControlBounds ( macScrollPtr->sbHandle, &contrlRect );
         }
         if (!(macScrollPtr->macFlags & FLUSH_LEFT)) {
             macScrollPtr->macFlags |= FLUSH_LEFT;
@@ -1092,6 +1082,13 @@ UpdateControlValues(
         TkMacOSXSetScrollbarGrow((TkWindow *) tkwin, false);
     }
 
+    /*
+     * Ensure we set scrollbar control bounds only once all size
+     * adjustments have been computed.
+     */
+
+    SetControlBounds(macScrollPtr->sbHandle, &contrlRect);
+
     /*
      * Given the Tk parameters for the fractions of the start and
      * end of the thumb, the following calculation determines the
