@@ -12,22 +12,10 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tkMacOSXXStubs.c,v 1.2.2.10 2005/11/30 01:02:55 hobbs Exp $
+ * RCS: @(#) $Id: tkMacOSXXStubs.c,v 1.2.2.13 2006/09/10 17:07:36 das Exp $
  */
 
-#include "tkInt.h"
-#include <X11/X.h>
-#include <X11/Xlib.h>
-#include <stdio.h>
-#include <stdlib.h>
-#include <pthread.h>
-#include <sys/types.h>
-
-#include <X11/Xatom.h>
-
-#include <Carbon/Carbon.h>
 #include "tkMacOSXInt.h"
-#include "tkPort.h"
 #include "tkMacOSXEvent.h"
 
 /*
@@ -666,6 +654,19 @@ XQueryColor(
     Colormap colormap,
     XColor* def_in_out)
 {
+    unsigned long p;
+    unsigned char r, g, b;
+    XColor *d = def_in_out;
+
+    p       	= d->pixel;
+    r       	= (p & 0x00FF0000) >> 16;
+    g       	= (p & 0x0000FF00) >> 8;
+    b       	= (p & 0x000000FF);
+    d->red  	= (r << 8) | r;
+    d->green	= (g << 8) | g;
+    d->blue 	= (b << 8) | b;
+    d->flags	= DoRed|DoGreen|DoBlue;
+    d->pad  	= 0;
 }
 
 void
@@ -675,6 +676,22 @@ XQueryColors(
     XColor* defs_in_out,
     int ncolors)
 {
+    int i;
+    unsigned long p;
+    unsigned char r, g, b;
+    XColor *d = defs_in_out;
+
+    for (i = 0; i < ncolors; i++, d++) {
+	p       	= d->pixel;
+	r       	= (p & 0x00FF0000) >> 16;
+	g       	= (p & 0x0000FF00) >> 8;
+	b       	= (p & 0x000000FF);
+	d->red  	= (r << 8) | r;
+	d->green	= (g << 8) | g;
+	d->blue 	= (b << 8) | b;
+	d->flags	= DoRed|DoGreen|DoBlue;
+	d->pad  	= 0;
+    }
 }
 
 int   
@@ -854,11 +871,11 @@ TkMacOSXXPutPixel(
     b  = (pixel & image->blue_mask);
     if (image->obdata) {
         /* Image from XGetImage, 16 bit color values */
-        cPix . red = r << 8;  
+        cPix . red = r << 8;
         cPix . green = g << 8;
         cPix . blue = b << 8;
     } else {
-        cPix . red = r;  
+        cPix . red = r;
         cPix . green = g;
         cPix . blue = b;
     }
@@ -965,7 +982,7 @@ XSetWindowColormap(
     Debugger();
 }
 
-Status		
+Status
 XStringListToTextProperty(
     char** list, 
     int count, 
